#===istalismanplugin===
# -*- coding: utf-8 -*-

#  Talisman plugin
#  pingX_plugin.py



import sys
import os
import struct
import array
import time
import select
import binascii
import math
import getopt
import string
import socket

# total size of data (payload)
ICMP_DATA_STR = 56  

# initial values of header variables
ICMP_TYPE = 8
ICMP_TYPE_IP6 = 128
ICMP_CODE = 0
ICMP_CHECKSUM = 0
ICMP_ID = 0
ICMP_SEQ_NR = 0

# Package definitions.
__program__   = 'ping'
__version__   = '0.6'
__date__      = '2005/07/08'
__author__    = 'Lars Strand <lars at unik no>/Modified by Marcin Jurczuk <spock at tkb dot pl>'
__licence__   = 'GPL'
__copyright__ = 'Copyright (C) 2004 Lars Strand'

def _construct(id, size, ipv6):
    """Constructs a ICMP echo packet of variable size
    """

    # size must be big enough to contain time sent
    if size < int(struct.calcsize("d")):
        _error("packetsize to small, must be at least %d" % int(struct.calcsize("d")))
    
    # construct header
    if ipv6:
        header = struct.pack('BbHHh', ICMP_TYPE_IP6, ICMP_CODE, ICMP_CHECKSUM, \
                             ICMP_ID, ICMP_SEQ_NR+id)
    else:
        header = struct.pack('bbHHh', ICMP_TYPE, ICMP_CODE, ICMP_CHECKSUM, \
                             ICMP_ID, ICMP_SEQ_NR+id)

    # if size big enough, embed this payload
    load = "-- IF YOU ARE READING THIS YOU ARE A NERD! --"
    
    # space for time
    size -= struct.calcsize("d")

    # construct payload based on size, may be omitted :)
    rest = ""
    if size > len(load):
        rest = load
        size -= len(load)

    # pad the rest of payload
    rest += size * "X"

    # pack
    data = struct.pack("d", time.time()) + rest
    packet = header + data          # ping packet without checksum
    checksum = _in_cksum(packet)    # make checksum

    # construct header with correct checksum
    if ipv6:
        header = struct.pack('BbHHh', ICMP_TYPE_IP6, ICMP_CODE, checksum, \
                             ICMP_ID, ICMP_SEQ_NR+id)
    else:
        header = struct.pack('bbHHh', ICMP_TYPE, ICMP_CODE, checksum, ICMP_ID, \
                             ICMP_SEQ_NR+id)

    # ping packet *with* checksum
    packet = header + data 

    # a perfectly formatted ICMP echo packet
    return packet

def _in_cksum(packet):


    # add byte if not dividable by 2
    if len(packet) & 1:              
        packet = packet + '\0'

    # split into 16-bit word and insert into a binary array
    words = array.array('h', packet) 
    sum = 0

    # perform ones complement arithmetic on 16-bit words
    for word in words:
        sum += (word & 0xffff) 

    hi = sum >> 16 
    lo = sum & 0xffff 
    sum = hi + lo
    sum = sum + (sum >> 16)
    
    return (~sum) & 0xffff # return ones complement


def pingNode(type, source, alive=0, timeout=1.0, ipv6=0, number=sys.maxint, node=None, flood=0, size=ICMP_DATA_STR,status_only=0):
    res = ''
    # if no node, exit

        
    # if not a valid host, exit
    if ipv6:
        if socket.has_ipv6:
            try:
                info, port = socket.getaddrinfo(node, None)
                host = info[4][0]
                # do not print ipv6 twice if ipv6 address given as node
                if host == node: 
                    noPrintIPv6adr = 1
            except:
                res +=  u'Не могу найти %s: Неизвестный хост' % node +'\n'
        else:
            res +=  u'Недоступно IPv6 для этой платформе' +'\n'
    else:    # IPv4
        try:
            host = socket.gethostbyname(node)
        except:
            res +=  u'Не могу найти %s: Неизвестный хост' % node +'\n'

    # trying to ping a network?
    if not ipv6:
        try:
            if int(string.split(host, ".")[-1]) == 0:
                res +=  u'Нет поддержки пинга в сети' +'\n'
        except:
            res +=  u'Пинг: ошибка, не корректный запрос' +'\n'
            host = '0.0.0.0'

    # do some sanity check
    if number == 0:
        res +=  u'Ошибка количества пакетов на передачу: %s' % str(a) +'\n'
    if alive:
        number = 1

    # Send the ping(s)
    start = 1; mint = 999; maxt = 0.0; avg = 0.0
    lost = 0; tsum = 0.0; tsumsq = 0.0

    # tell the user what we do
    if not alive:
        if ipv6 and not status_only:
            # do not print the ipv6 twice if ip adress given as node
            # (it can be to long in term window)
            if noPrintIPv6adr == 1:
                # add 40 (header) + 8 (icmp header) + payload
                res += u'Пинг: %s : %d байты (40+8+%d)' % (str(node), 40+8+size, size) +'\n'
                #reply(type, source, u'PING %s : %d data bytes (40+8+%d)' % (str(node), \
                #                                             40+8+size, size))
            else:
                # add 40 (header) + 8 (icmp header) + payload
                res += u'Пинг: %s (%s): %d байты (40+8+%d)' % (str(node), str(host), 40+8+size, size) +'\n'
                #reply(type, source, u'PING %s (%s): %d data bytes (40+8+%d)' % (str(node), \
                #                                                 str(host), 40+8+size, size))
        elif not status_only:
            # add 20 (header) + 8 (icmp header) + payload
            #reply(type, source, u'PING %s (%s): %d data bytes (20+8+%d)' % (str(node), str(host), \
            #                                                 20+8+size, size))
            res +=  u'Пинг: %s (%s): %d байты (20+8+%d)' % (str(node), str(host), 20+8+size, size) +'\n'
	else:
		pass
        
    # trap ctrl-d and ctrl-c
    try:
        
        # send the number of ping packets as given
        while start <= number:
            lost += 1 # in case user hit ctrl-c
            
            # create the IPv6/IPv4 socket
            if ipv6:
                # can not create a raw socket if not root or setuid to root
                try:
                    pingSocket = socket.socket(socket.AF_INET6, socket.SOCK_RAW, \
                                               socket.getprotobyname("ipv6-icmp"))
                except socket.error, e:
                    res +=  u'Ошибка сокета: %s' % e + u' You must be root (%s uses raw sockets)' % os.path.basename(sys.argv[0]) +'\n'
                    
            # IPv4
            else:
                # can not create a raw socket if not root or setuid to root
                try:
                    pingSocket = socket.socket(socket.AF_INET, socket.SOCK_RAW, \
                                               socket.getprotobyname("icmp"))
                except socket.error, e:
                    res +=  u'Ошибка сокета: %s' % e + u' You must be root (%s uses raw sockets)' % os.path.basename(sys.argv[0]) +'\n'
                 
            packet = _construct(start, size, ipv6) # make a ping packet

            # send the ping
            try:
                pingSocket.sendto(packet,(node,1))
            except socket.error, e:
                res +=  u'Ошибка сокета: %s' % e +'\n'

            # reset values
            pong = ""; iwtd = []

            # wait until there is data in the socket
            while 1:
                # input, output, exceptional conditions
                iwtd, owtd, ewtd = select.select([pingSocket], [], [], timeout)
                break # no data and timout occurred 

            # data on socket - this means we have an answer
            if iwtd:  # ok, data on socket
                endtime = time.time()  # time packet received
                # read data (we only need the header)
                pong, address = pingSocket.recvfrom(size+48)
                lost -= 1 # in case user hit ctrl-c

                # examine packet
                # fetch TTL from IP header
                if ipv6:
                    # since IPv6 header and any extension header are never passed
                    # to a raw socket, we can *not* get hoplimit field..
                    # I hoped that a socket option would help, but it's not
                    # supported:
                    #   pingSocket.setsockopt(IPPROTO_IPV6, IPV6_RECVHOPLIMIT, 1)
                    # so we can't fetch hoplimit..

                    # fetch hoplimit
                    #rawPongHop = struct.unpack("c", pong[7])[0]

                    # fetch pong header
                    pongHeader = pong[0:8]
                    pongType, pongCode, pongChksum, pongID, pongSeqnr = \
                              struct.unpack("bbHHh", pongHeader)

                    # fetch starttime from pong
                    starttime = struct.unpack("d", pong[8:16])[0]

                # IPv4
                else:
                    # time to live
                    rawPongHop = struct.unpack("s", pong[8])[0]

                    # convert TTL from 8 bit to 16 bit integer
                    pongHop = int(binascii.hexlify(str(rawPongHop)), 16)

                    # fetch pong header
                    pongHeader = pong[20:28]
                    pongType, pongCode, pongChksum, pongID, pongSeqnr = \
                              struct.unpack("bbHHh", pongHeader)

                    # fetch starttime from pong
                    starttime = struct.unpack("d", pong[28:36])[0]

                # valid ping packet received?
                if not pongSeqnr == start:
                    pong = None

            # NO data on socket - timeout waiting for answer
            if not pong:
                if alive and not status_only:
                    res +=  u'Нет ответа от %s (%s)' % (str(node), str(host)) +'\n'
		elif alive and status_only:
		    return (1)
                else:
                    res +=  u'Пинг таймаут: %s (icmp_seq=%d) ' % (host, start) +'\n'

                # do not wait if just sending one packet
                if number != 1 and start < number:
                    time.sleep(flood ^ 1)
                start += 1
                continue  # lost a packet - try again

            triptime  = endtime - starttime # compute RRT
            tsum     += triptime            # triptime for all packets (stddev)
            tsumsq   += triptime * triptime # triptime^2  for all packets (stddev)

            # compute statistic
            maxt = max ((triptime, maxt))
            mint = min ((triptime, mint))

            if alive and not status_only:
                res +=  str(node) + ' (' + str(host) +u') жив' +'\n'
	    elif alive and status_only:
		return (0)
            else:
                if ipv6:
                    # size + 8 = payload + header
                    res += u'%d байт от %s: #=%d время=%.5f ms' % \
                          (size+8, host, pongSeqnr, triptime*1000) +'\n'
                else:
                        res += u'%d байт от %s: #=%d ttl=%s время=%.5f мс' % (size+8, host, pongSeqnr, pongHop, triptime*1000) + '\n'
                    #reply(type, source, u'%d bytes from %s: icmp_seq=%d ttl=%s time=%.5f ms' % \
                    #      (size+8, host, pongSeqnr, pongHop, triptime*1000))

            # do not wait if just sending one packet
            if number != 1 and start < number:
                # if flood = 1; do not sleep - just ping                
                time.sleep(flood ^ 1) # wait before send new packet

            # the last thing to do is update the counter - else the value
            # (can) get wrong when computing summary at the end (if user
            # hit ctrl-c when pinging)
            start += 1
            # end ping send/recv while
        
    # if user ctrl-d or ctrl-c
    except (EOFError, KeyboardInterrupt):
        # if user disrupts ping, it is most likly done before
        # the counter get updates - if do not update it here, the
        # summary get all wrong.
        start += 1
        pass

    # compute and print som stats
    # stddev computation based on ping.c from FreeBSD
    if start != 0 or lost > 0:  # do not print stats if 0 packet sent
        start -= 1              # since while is '<='
        avg = tsum / start      # avg round trip
        vari = tsumsq / start - avg * avg 
        # %-packet lost
        if start == lost:
            plost = 100
        else:
            plost = (lost/start)*100

        if not alive:
            res += u'\n--- %s статистика пинга ---' % node +'\n'
            res += u'%d пакетов отправлено, %d пакетов принято, %d%% потеряно пакетов' % (start, start-lost, plost) +'\n'
            #reply(type, source, u'\n--- %s ping statistics ---' % node)
            #reply(type, source, u'%d packets transmitted, %d packets received, %d%% packet loss' % \
            #      (start, start-lost, plost))
            # don't display summary if 100% packet-loss
            if plost != 100:
                res += u'Итог мин./сред./макс./разн. = %.3f/%.3f/%.3f/%.3f мс' % (mint*1000, (tsum/start)*1000, maxt*1000, math.sqrt(vari)*1000) +'\n'
                #reply(type, source, u'round-trip min/avg/max/stddev = %.3f/%.3f/%.3f/%.3f ms' % \
                #      (mint*1000, (tsum/start)*1000, maxt*1000, math.sqrt(vari)*1000))
    reply(type, source, res)
    pingSocket.close()

def ping_netping_start(type, source, parameters):
        if parameters == '':
            reply(type, source, u'Что пингуем???')
            return
        if __name__ == '__main__':



                node = parameters.split(' ')[0]
                if parameters.count('-a'):
                        alive = 1
                else:
                        alive = 0
                if parameters.count('-c'):
                        try:
                                a = parameters.split('-c=')
                                b = a[1].split(' ')[0]
                                count = int(b)
                        except:
                                count = 3
                else:
                        count = 3
                timeout = 1.0; ipv6 = 0;
                flood = 0; size = ICMP_DATA_STR


                pingNode(type, source, alive=alive, timeout=timeout, ipv6=ipv6, number=count, node=node, flood=flood, size=size)

                sys.exit(0)


register_command_handler(ping_netping_start, 'netping', ['админ','new','все'], 10, 'NET PING. Пингует любой адрес/IP-адрес', 'netping <адрес> [ключи]', ['netping 127.0.0.1', 'netping 127.0.0.1 -a', 'netping 127.0.0.1 -c=5'])
